@@ -17,7 +17,6 @@
  * this warranty disclaimer.
  */
 
-#include <linux/iopoll.h>
 #include <linux/firmware.h>
 
 #include "decl.h"
@@ -430,7 +429,6 @@ static void mwifiex_pcie_remove(struct pci_dev *pdev)
 	struct mwifiex_private *priv;
 	const struct mwifiex_pcie_card_reg *reg;
 	u32 fw_status;
-	int ret;
 
 	card = pci_get_drvdata(pdev);
 
@@ -442,7 +440,7 @@ static void mwifiex_pcie_remove(struct pci_dev *pdev)
 
 	reg = card->pcie.reg;
 	if (reg)
-		ret = mwifiex_read_reg(adapter, reg->fw_status, &fw_status);
+		mwifiex_read_reg(adapter, reg->fw_status, &fw_status);
 	else
 		fw_status = -1;
 
@@ -572,11 +570,7 @@ static void mwifiex_pcie_reset_prepare_notify(struct pci_dev *dev,
 }
 #endif
 
-static 
-#if LINUX_VERSION_IS_GEQ(3,7,0)
-const 
-#endif /* LINUX_VERSION_IS_GEQ(3,7,0) */
-struct pci_error_handlers mwifiex_pcie_err_handler = {
+static const struct pci_error_handlers mwifiex_pcie_err_handler = {
 #if LINUX_VERSION_IS_GEQ(4,13,0)
 	.reset_prepare		= mwifiex_pcie_reset_prepare,
 	.reset_done		= mwifiex_pcie_reset_done,
@@ -660,15 +654,11 @@ static void mwifiex_delay_for_sleep_cookie(struct mwifiex_adapter *adapter,
 			    "max count reached while accessing sleep cookie\n");
 }
 
-#define N_WAKEUP_TRIES_SHORT_INTERVAL 15
-#define N_WAKEUP_TRIES_LONG_INTERVAL 35
-
 /* This function wakes up the card by reading fw_status register. */
 static int mwifiex_pm_wakeup_card(struct mwifiex_adapter *adapter)
 {
 	struct pcie_service_card *card = adapter->card;
 	const struct mwifiex_pcie_card_reg *reg = card->pcie.reg;
-	int retval;
 
 	mwifiex_dbg(adapter, EVENT,
 		    "event: Wakeup device...\n");
@@ -676,24 +666,11 @@ static int mwifiex_pm_wakeup_card(struct mwifiex_adapter *adapter)
 	if (reg->sleep_cookie)
 		mwifiex_pcie_dev_wakeup_delay(adapter);
 
-	/* The 88W8897 PCIe+USB firmware (latest version 15.68.19.p21) sometimes
-	 * appears to ignore or miss our wakeup request, so we continue trying
-	 * until we receive an interrupt from the card.
-	 */
-	if (read_poll_timeout(mwifiex_write_reg, retval,
-			      READ_ONCE(adapter->int_status) != 0,
-			      500, 500 * N_WAKEUP_TRIES_SHORT_INTERVAL,
-			      false,
-			      adapter, reg->fw_status, FIRMWARE_READY_PCIE)) {
-		if (read_poll_timeout(mwifiex_write_reg, retval,
-				      READ_ONCE(adapter->int_status) != 0,
-				      10000, 10000 * N_WAKEUP_TRIES_LONG_INTERVAL,
-				      false,
-				      adapter, reg->fw_status, FIRMWARE_READY_PCIE)) {
-			mwifiex_dbg(adapter, ERROR,
-				    "Firmware didn't wake up\n");
-			return -EIO;
-		}
+	/* Accessing fw_status register will wakeup device */
+	if (mwifiex_write_reg(adapter, reg->fw_status, FIRMWARE_READY_PCIE)) {
+		mwifiex_dbg(adapter, ERROR,
+			    "Writing fw_status register failed\n");
+		return -1;
 	}
 
 	if (reg->sleep_cookie) {
@@ -1272,7 +1249,7 @@ static int mwifiex_pcie_delete_cmdrsp_buf(struct mwifiex_adapter *adapter)
 static int mwifiex_pcie_alloc_sleep_cookie_buf(struct mwifiex_adapter *adapter)
 {
 	struct pcie_service_card *card = adapter->card;
-	u32 *cookie;
+	u32 tmp;
 
 	card->sleep_cookie_vbase = dma_alloc_coherent(&card->dev->dev,
 						      sizeof(u32),
@@ -1283,11 +1260,13 @@ static int mwifiex_pcie_alloc_sleep_cookie_buf(struct mwifiex_adapter *adapter)
 			    "dma_alloc_coherent failed!\n");
 		return -ENOMEM;
 	}
-	cookie = (u32 *)card->sleep_cookie_vbase;
 	/* Init val of Sleep Cookie */
-	*cookie = FW_AWAKE_COOKIE;
+	tmp = FW_AWAKE_COOKIE;
+	put_unaligned(tmp, card->sleep_cookie_vbase);
 
-	mwifiex_dbg(adapter, INFO, "alloc_scook: sleep cookie=0x%x\n", *cookie);
+	mwifiex_dbg(adapter, INFO,
+		    "alloc_scook: sleep cookie=0x%x\n",
+		    get_unaligned(card->sleep_cookie_vbase));
 
 	return 0;
 }
@@ -1520,14 +1499,6 @@ mwifiex_pcie_send_data(struct mwifiex_adapter *adapter, struct sk_buff *skb,
 			ret = -1;
 			goto done_unmap;
 		}
-
-		/* The firmware (latest version 15.68.19.p21) of the 88W8897 PCIe+USB card
-		 * seems to crash randomly after setting the TX ring write pointer when
-		 * ASPM powersaving is enabled. A workaround seems to be keeping the bus
-		 * busy by reading a random register afterwards.
-		 */
-		mwifiex_read_reg(adapter, PCI_VENDOR_ID, &rx_val);
-
 		if ((mwifiex_pcie_txbd_not_full(card)) &&
 		    tx_param->next_pkt_len) {
 			/* have more packets and TxBD still can hold more */
@@ -3189,7 +3160,6 @@ static void mwifiex_cleanup_pcie(struct mwifiex_adapter *adapter)
 	struct pcie_service_card *card = adapter->card;
 	struct pci_dev *pdev = card->dev;
 	const struct mwifiex_pcie_card_reg *reg = card->pcie.reg;
-	int ret;
 	u32 fw_status;
 
 	/* Perform the cancel_work_sync() only when we're not resetting
@@ -3206,7 +3176,7 @@ static void mwifiex_cleanup_pcie(struct mwifiex_adapter *adapter)
 			    "skipped cancel_work_sync() because we're in card reset failure path\n");
 	}
 
-	ret = mwifiex_read_reg(adapter, reg->fw_status, &fw_status);
+	mwifiex_read_reg(adapter, reg->fw_status, &fw_status);
 	if (fw_status == FIRMWARE_READY_PCIE) {
 		mwifiex_dbg(adapter, INFO,
 			    "Clearing driver ready signature\n");
